from linusRes import RESIDUES, Contact_Energies, SASA

def set_solvation_parameters(p, probe=1.4):
    fai = p.residue_first_atom_indices
    rn = p.residue_names
    satoms = []
    for i in range(p.num_residues):
        first = fai[i]
        last = fai[i+1]
        sd = SASA[rn[i]]
        for j in range(first, last):
            atom = p.atoms[j]
            atom.sasarad, atom.sasapar = sd[atom.name]
            atom.sasarad = atom.sasarad + probe
            satoms.append(atom)
    return satoms


def make_contact_list(p, wmin, wmax, cscale=1.0):
    fai = p.residue_first_atom_indices
    rn = p.residue_names
    nr = range(1, p.num_residues)

    def get_atom_contact_radius(atom_name, res):
        for arec in res:
            if arec[0] == atom_name: return arec[5]

    is_contact_pair = Contact_Energies.has_key

    clist = []

    for i in nr:
        rname = rn[i]
        start = fai[i]
        end = fai[i+1]
        res = RESIDUES[rname]
        for a in range(start, end):
            atom = p.atoms[a]
            arad = get_atom_contact_radius(atom.name, res)
            if arad <= 1.0e-6: continue
            for j in range(i+wmin, min(nr[-1], i+wmax)+1):
                jrname = rn[j]
                if is_contact_pair((rname, jrname)):
                    econ = Contact_Energies[(rname, jrname)] * cscale
                elif is_contact_pair((jrname, rname)):
                    econ = Contact_Energies[(jrname, rname)] * cscale
                else:
                    continue
                jstart = fai[j]
                jend = fai[j+1]
                jres = RESIDUES[jrname]
                for b in range(jstart, jend):
                    jatom = p.atoms[b]
                    brad = get_atom_contact_radius(jatom.name, jres)
                    if brad >= 1.0e-6:
                        clist.append((atom, jatom, arad+brad, econ))
     
    return clist

def get_atom_named(atoms, atom_name, start, end):
    for i in range(start, end):
        atom = atoms[i]
        if atom.name == atom_name:
            return atom

def get_scacceptor(atoms, start, end, res):
    scatoms = []
    for arec in RESIDUES[res]:
        if arec[7] and arec[0] <> ' O  ':
            scatoms.append(get_atom_named(atoms, arec[0], start, end))
    return scatoms

def make_hbond_list(p, wmin, wmax, hbparms):
    fai = p.residue_first_atom_indices
    rn = p.residue_names
    nr = range(p.num_residues)
    atoms = p.atoms

    hblist = []
    hbdist = hbparms['hbond_distance']
    hbdmax = hbdist + hbparms['hbond_probe']
    hbdtor = hbparms['hbond_torsion']
    hbenes = hbparms['hbond_score_short']
    hbenel = hbparms['hbond_score_long']
    shbdis = hbparms['sidechain_hbond_distance']
    shbtor = hbparms['sidechain_hbond_torsion']
    shbene = hbparms['sidechain_hbond_score']

    for i in range(1, p.num_residues):
        start = fai[i]
        end = fai[i+1]
        if rn[i] <> 'PRO':
            donor = get_atom_named(atoms, ' N  ', start, end)
            da1 = get_atom_named(atoms, ' CA ', start, end)
            da2 = get_atom_named(atoms, ' C  ', fai[i-1], start)
            acps = []
            for j in range(wmin, wmax):
                pos = i-j
                if pos >= 0:
                    if hbparms['use_hbond']:
                        acp = get_atom_named(atoms, ' O  ', fai[pos],
                                             fai[pos+1])
                        if j < 6:
                            acps.append((acp, hbdist, hbdmax, hbdtor, hbenes))
                        else:
                            acps.append((acp, hbdist, hbdmax, hbdtor, hbenel))
                    if hbparms['use_sidechain_hbond']:
                        scatoms = get_scacceptor(atoms, fai[pos], fai[pos+1],
                                                 rn[pos])
                        if scatoms:
                            for acp in scatoms:
                                acps.append((acp, shbdis, shbdis, shbtor,
                                             shbene))

                pos = i+j
                if pos < p.num_residues:
                    if hbparms['use_hbond']:
                        acp = get_atom_named(atoms, ' O  ', fai[pos],
                                            fai[pos+1])
                        if j < 6:
                            acps.append((acp, hbdist, hbdmax, hbdtor, hbenes))
                        else:
                            acps.append((acp, hbdist, hbdmax, hbdtor, hbenel))
                    if hbparms['use_sidechain_hbond']:
                        scatoms = get_scacceptor(atoms, fai[pos], fai[pos+1],
                                                 rn[pos])
                        if scatoms:
                            for acp in scatoms:
                                acps.append((acp, shbdis, shbdis, shbtor,
                                             shbene))
            hblist.append(((donor, da1, da2), tuple(acps)))

    return hblist

def print_hbond_list(p, hblist, file):
    if type(file) is type(''):
        ff = open(file, 'wb')
    else:
        ff = file

    Write = ff.write
    fai = p.residue_first_atom_indices
    rn = p.residue_names
    atoms = p.atoms

    atom_ids = {}

    for i in range(p.num_residues):
        start = fai[i]
        end = fai[i+1]
        for j in range(start, end):
            atom_ids[id(atoms[j])] = (i, rn[i])

    fmt = "%5d %3s %4s %5d %3s %4s Distance = %.3f to %.3f Score = %.3f\n"

    for rec in hblist:
        donor = rec[0][0]
        r1, rnam1 = atom_ids[id(donor)]
        for acps in rec[1]:
            acp = acps[0]
            r2, rnam2 = atom_ids[id(acp)]
            Write(fmt % (r1+1, rnam1, donor.name, r2+1, rnam2, acp.name,
                         acps[1], acps[2], acps[4]))
    if ff <> file:
        ff.close()
        
            


                
        
