#!/usr/bin/env python
"""Write out a FASTA style sequence file by parsing the ATOM records
in a PDB file"""

USAGE = """
python pdbATOM2fasta.py pdb[code].ent 
Output is in file pdb[code].fasta
"""

import pdbparse
import sys,string,os

seq3to1 = {'ALA': 'A', 'ARG': 'R', 'ASN': 'N', 'ASP': 'D', 'CYS': 'C',
           'CYX': 'C', 'GLN': 'Q', 'GLU': 'E', 'GLY': 'G', 'HIS': 'H',
           'ILE': 'I', 'LEU': 'L', 'LYS': 'K', 'MET': 'M', 'PHE': 'F',
           'PRO': 'P', 'SER': 'S', 'THR': 'T', 'TRP': 'W', 'TYR': 'Y',
           'VAL': 'V', 'ASX': 'B', 'GLX': 'Z'}


def run(infile):
    mol = pdbparse.read_pdb(infile)
    mol.delete_hetero()
#   print mol.name
    basename = os.path.basename(infile)
    prefix = os.path.splitext(basename)[0]
    seqfile = open(prefix + '.fasta', 'w')
    seq = []
    for chain in mol:
        if chain.type() <> 'protein': continue
        chain.gaps()
        for res in chain.elements:
            aa = res.name
            seq.append(seq3to1.get(aa, aa) )

    seqfile.write('>%s %s\n' % (mol.name, len(seq)))
    seq = string.join(seq, '')
    for i in range(0, len(seq), 60):
        for k in range(i, i+60, 10):
            subseq = seq[k:k+10]
            if not subseq:
                break
            seqfile.write('%s ' % subseq)
        seqfile.write('\n')

    seqfile.close()
if __name__ == "__main__":

    if len(sys.argv) < 2:
        print USAGE
        sys.exit()

    infile = sys.argv[1]

    run(infile)
