#sort_sc.py
#Lauren Perskie
#8/29/07
#This module attempts to find the best possible configuration of sidechains
#with their backbone donors/acceptors.
#It works as follows.
#
#sort_sc_pos is passed a matrix of 1's and 0's such as:
#
#            [1 1 0 0]
#            [0 1 0 0]
#            [0 0 1 1]
#            [0 1 1 1]
#
#First the row with the least number of 1s is found, and then the column
#in that row with the least number of 1s is found.  In this case, it would be
#row 2, column 2.  Now all entries column 2 except row 2 and all entries in row
#2 except column 2 are set to 0.
#
#            [1 0 0 0]
#            [0 1 0 0]
#            [0 0 1 1]
#            [0 0 1 1]
#
#This process is repeated until each row contains a single 1.
#If all rows contain a single 1 at the end, they are all satisfied.
#The end result here would be:
#
#            [1 0 0 0]
#            [0 1 0 0]
#            [0 0 1 0]
#            [0 0 0 1]
#
#There are cases where there are not enough sidechains to satisfy the backbone
#In that case, there will be rows of zeros.  Here is an example.
#Starting matrix:
#
#            [0 1 0 0]
#            [0 1 0 0]
#            [0 0 0 1]
#            [0 0 0 1]
#
#Will end up as:
#
#            [0 1 0 0]
#            [0 0 0 0]
#            [0 0 0 1]
#            [0 0 0 0]
#
#If each row contains a 1 at the end of the calculation, a 1 is returned
#(meaning that all bb donors/acceptors were satisfied).
#Otherwise a 0 is returned.



import Numeric as N

        
def sort_sc_pos(pos_mat):

    satisfied = 1;

    rowIdxs = range(len(pos_mat));

    for i in xrange(len(rowIdxs)):

        rowSums = [sum(pos_mat[x]) for x in rowIdxs];
        minRowIdx = rowIdxs[findMinIdx(rowSums)];
        
        if min(rowSums) == 0:
            satisfied = 0;
            rowIdxs.remove(minRowIdx);
            continue
        
        colSums = [sum(pos_mat[:,x]) for x in xrange(len(pos_mat[minRowIdx]))];
        minColIdx = findMinColIdx(pos_mat,minRowIdx);
        setPos(pos_mat,minRowIdx,minColIdx);
        rowIdxs.remove(minRowIdx);
        
    return satisfied;

def findMinIdx(vals):

    minsum = min(vals);

    for i in xrange(len(vals)):
        if vals[i] == minsum:
            return i;

def findMinColIdx(pos_mat,minRowIdx):
    
    row_ones = [x for x in xrange(len(pos_mat[minRowIdx]))
                if pos_mat[minRowIdx][x] == 1]
    allowedCols = [sum(pos_mat[:,x]) for x in row_ones]

    return row_ones[findMinIdx(allowedCols)];

def setPos(pos_mat,minRowIdx,minColIdx):

    pos_mat[minRowIdx] = N.zeros(len(pos_mat[minRowIdx]));
    pos_mat[:,minColIdx] = N.zeros(len(pos_mat[:,minColIdx]));
    pos_mat[minRowIdx][minColIdx] = 1;

    return
    
    
