import time
import linusScore
import os

__version__ = 0.1

def run(sim):
    from linusSetConf import choose_conformation, set_coil, set_super_helix
    from bumpcheck import bump_check, bump_check_intra
    from linusSecStr import rc_gen, rc_ss
    from pdbio import write_pdb
    from math import exp
    from Atom3d import ztox, commit_internal_coords, restore_internal_coords
    #from find_hbonds import hbondLists,hbondSatisfied,hbondInit 

    # get all necessary files
    
    logfile = sim.files.get_logfile()
    minfile= sim.files.get_min_file()
    outfile = sim.files.get_out_file()
    rcfile = sim.files.get_rc_file()
    stpfile = sim.files.create_filename('stp')

    # write out simulation parameters etc
    
    writeprologue(logfile, sim)

    resrange = range(sim.protein.num_residues)

    # list in which the number, type and success of moves
    # attempted for each residue is to be stored

    move_types_attempted = []
    for i in resrange:
        move_types_attempted.append([[0, 0], [0, 0], [0, 0], [0, 0], [0, 0], [0, 0]])

    # list in which the secondary structure information for
    # each residue is stored

    ss_list = []
    for i in resrange:
        ss_list.append([0, 0, 0, 0, 0, 0])

    # list of residues whose conformation will be changed

    moveres = []
    for i in resrange:
        if sim.movelist[i]:
            moveres.append(i)
    
    # counters for number of structures accepted and rejected

    nacp = nrej = nrej_steric = nrej_hbs = 0

    # temporary array in which to store conformation codes

    rcarray = ['?']*sim.protein.num_residues
    ssarray = ['C']*sim.protein.num_residues

    # aliases for various attributes/functions
    
    protein = sim.protein
    numres = protein.num_residues
    numatm = protein.num_atoms
    fai = protein.residue_first_atom_indices
    atoms = protein.atoms
    use_solvation = sim.use_solvation
    nsave = sim.numsave
    beta = sim.beta
    hlx_w = protein.residue_helix_weights
    sht_w = protein.residue_strand_weights
    #hbondParams = sim.hbondParams

    user_functions = sim.registered_functions
    generator = sim.generator
    choice = sim.generator.choice
    random = sim.generator.random
    total_score = sim.total_score
    Apply = apply
    exists = os.path.exists
    unlink = os.unlink
    
    code_types = {'H': 0, 'E': 1, '1': 2, '2': 3, 'C': 4, 'P': 5}

    nattempts = range(1000)

    #hbond initialization
    
    #hbondInit(protein,hbondParams,hbondLists);

    #hbond aliases
    inc = float(numres)/sim.numcycle;
    
    # compute scored of intital conformation

    eref = ebest = total_score()

    for cyc in xrange(1, sim.numcycle+1):
        for XXX in moveres:
            
            # choose residue number whose conformation is to be changed
            res = choice(moveres)

            # default is to change conformation of res-1, res and res+1
            start = res - 1
            end = res + 2

            # choose move type
            code, func = choose_conformation(protein, res, generator)

            # if move type is helix or strand extend the list of residues,
            # whose conformation is to be changed, in either direction till
            # a residue whose weight for the move is less than 0.4

            if code == 'H' or code == 'E':
                ##if code == 'H':
                    ##w1 = hlx_w
                    ##w2 = sht_w
                ##else:
                    ##w1 = sht_w
                    ##w2 = hlx_w
                ##for j in range(start-1, -1, -1):
                    ##if w1[j] > 0.4 and w2[j] <= 0.4:
                        ##start = j
                    ##else:
                        ##break
                ##for j in range(end, numres):
                    ##if w1[j] > 0.4 and w2[j] <= 0.4:
                        ##end = j+1
                    ##else:
                        ##break
                ##if end - start >= 5 and code == 'H':
                    ##func = set_super_helix
                todo = ((func, (protein, start, end, generator)),)
            elif code == 'C':
                todo = ((func, (protein, start, end, generator)),)
            elif code == 'P':
                todo = ((func, (protein, start, end, generator)),)
            elif code == '1':
                todo = ((set_coil, (protein, start, res, generator)),
                        (func, (protein, res, res+1, generator)))
            elif code == '2':
                todo = ((func, (protein, start, res, generator)),
                        (set_coil, (protein, res+1, end, generator)))

            success = 0

            r_ids = range(start, end)
            fai_start = max(fai[start], 0)
            fai_end = min(fai[end], numatm)
            ztox(atoms, 0, fai_start)

            for at in nattempts:
                for f, arg in todo:
                    Apply(f, arg)
                ztox(atoms, fai_start, fai_end)
                int_ok = 1
                for r_id in r_ids:
                    if bump_check_intra(atoms, fai, r_id):
                        int_ok = 0
                        break
                if int_ok and \
                     (not bump_check(atoms, fai, start, end-1, start+1, end)):
                    success = 1
                    break

            if not success:
                restore_internal_coords(atoms, fai_start, fai_end)
                nrej_steric = nrej_steric + 1
                row = move_types_attempted[res][code_types[code]]
                row[1] = row[1] + 1
                continue

            if bump_check(atoms, fai, 0, start, start, end):
                restore_internal_coords(atoms, fai_start, fai_end)
                nrej_steric = nrej_steric + 1
                row = move_types_attempted[res][code_types[code]]
                row[1] = row[1] + 1
                continue

            ztox(atoms, fai_end)
            
            if bump_check(atoms, fai, 0, end, end, numres):
                restore_internal_coords(atoms, fai_start, fai_end)
                nrej_steric = nrej_steric + 1
                row = move_types_attempted[res][code_types[code]]
                row[1] = row[1] + 1
                continue
            #elif not hbondSatisfied(protein,start-int(inc*(cyc)),
             #                       end+int(inc*(cyc)),cyc,
              #                      hbondParams,hbondLists):
               # nrej_hbs = nrej_hbs+1
                #restore_internal_coords(atoms, fai_start, fai_end)
                #row = move_types_attempted[res][code_types[code]]
                #row[1] = row[1] + 1                            
                #continue
            else:
                
                ecur = total_score()
                if exp(beta * (eref-ecur)) > random():
                    eref = ecur
                    commit_internal_coords(atoms, fai_start, fai_end)
                    nacp = nacp + 1
                    row = move_types_attempted[res][code_types[code]]
                    row[0] = row[0] + 1
                    if ecur < ebest:
                        ebest = ecur
                        title = 'New Minima Cycle %d Energy %g' % (cyc,
                                                                   ebest)
                        write_pdb(protein, minfile, title)
                    
                else:
                    nrej = nrej + 1
                    restore_internal_coords(atoms, fai_start, fai_end)
                    row = move_types_attempted[res][code_types[code]]
                    row[1] = row[1] + 1

        # run any user specified functions
        ztox(atoms)
        for func, when in user_functions:
            if when == 1:
                func(sim)
        
        # generate and write structure codes for current structure
        rc = rc_gen(protein, rcarray)
        if sim.write_rc:
            rcfile.write(rc)
            rcfile.write(' %s\n' % eref)

        # assign secondary structure and update counters
        sst = rc_ss(rc, ssarray)
        for k in resrange:
            row = ss_list[k]
            c = code_types[sst[k]]
            row[c] = row[c] + 1

        # if time to write structure to file do it
        if not cyc % nsave:
            title = 'CYCLE %d ENERGY %g' % (cyc, eref)
            write_pdb(protein, outfile, title)

        # check if the simulation is to be aborted and if so terminate
        # gracefully

        if exists(stpfile):
            unlink(stpfile)
            break

    try:
        del sim.move_types_attempted
    except:
        pass

    sim.move_types_attempted = move_types_attempted
    writeepilogue(logfile, sim, nacp, nrej, nrej_steric, nrej_hbs, eref, ebest)
    swt = compute_weights(ss_list)
    wmax = max(sim.hbond_winmax, sim.contact_winmax)
    write_swt(sim.files.get_swt_filename(wmax, sim.trial),
              protein.residue_names, swt)
    plot_swt(sim.files.get_ps_filename(wmax, sim.trial),
              swt, protein.name+' Window Size = '+`wmax`)

    try:
        del sim.swt
    except AttributeError:
        pass
    sim.swt = swt
    ztox(atoms)
             
import bumpcheck
    
def list_bumps_and_energies(file, sim):
    from Atom3d import ztox
    log = file.write
    # steric clashes info
    
    log('\n----- Bump Check Information -----\n')
    bumpcheck.print_all_bumps(sim.protein, file)
    

    # energy parameters

    e = 0.0
    if sim.use_contact:
        log('\n----- Contact Score Information -----\n')
        log('Contact probe size: %g\n' % sim.contact_probe)
        log('Total number of contacts: %d\n' % len(sim.conlist))
        e = e + linusScore.print_contact_score(sim.protein, sim.conlist,
                                               sim.contact_probe, file)

    if sim.use_hbond or sim.use_sidechain_hbond:
        log('\n----- Hydrogen Bond Score Information -----\n')
        if sim.use_hbond:
            log('Hydrogen Bond Distance:   %g\n' % sim.hbond_distance)
            log('Hydrogen Bond Torsion:    %g\n' % sim.hbond_torsion)
            log('Hydrogen Bond Probe Size: %g\n' % sim.hbond_probe)
            log('Hydrogen Bond Score:\n\tShort: %g\n\t Long: %g\n' % (
                 sim.hbond_score_short, sim.hbond_score_long))
        if sim.use_sidechain_hbond:
            log('Sidechain Hydrogen Bond Distance: %g\n' % 
                 sim.sidechain_hbond_distance)
            log('Sidechain Hydrogen Bond Torsion:  %g\n' % 
                 sim.sidechain_hbond_torsion)
            log('Sidechain Hydrogen Bond Score:    %g\n' % 
                 sim.sidechain_hbond_score)

        e = e + linusScore.print_hbond_score(sim.protein, sim.hblist, file)

    if sim.use_torsion:
        log('\n----- Torsion Penalty Score Information -----\n')
        log('Torsion Penalty: %g\n' % sim.torsion_penalty)
        e = e + linusScore.print_torsion_score(sim.protein,
                                               sim.torsion_penalty,
                                               file)

    if sim.use_compact:
        log('\n----- Compaction Score Information -----\n')
        log('Compaction Fraction: %g\n' % sim.compact_fraction)
        e = e + linusScore.print_compaction_score(sim.protein, 
                                                  sim.compact_fraction,
                                                  file)

    if sim.use_solvation:
        log('\n----- Solvation Score Information -----\n')
        log('\tCavity Term: %g\n' % sim.cavity_term)
        e = e + linusScore.print_solvation_score(sim.protein,
                                                 sim.cavity_term,
                                                 sim.solvation_winmax,
                                                 file)

    log('\nTotal Energy of Initial Structure = %g\n' % e)
    

def writeprologue(file, sim):

    log = file.write
    header = '='*60 +'\n'
    if sim.trial == 1:
        log(header)
        log('LINUS version %s\n' % __version__)
        log(header)
        log('\nSimulation starting time: %s\n' % time.ctime(time.time()))

        # protein information 

        log('\nInput PDB Filename: %s\n' % sim.files.input)
        log('Molecule %s\n' % sim.protein.name)
        log('\tNumber of Residues: %d\n' % sim.protein.num_residues)
        log('\tNumber of Atoms   : %d\n' % sim.protein.num_atoms)

        # simulation parameters

        log('\n----- Simulation Parameters -----\n')
        log('\tNumber of Cycles: %12d\n' % sim.numcycle)
        log('\tStructure Save Frequency: %12d\n' % sim.numsave)
        log('\tRandom Generator Seeds at Start: %s\n' %
            `sim.generator.get_seed()`)
        log('\tBeta for run: %g\n' % sim.beta)
        log('\tNumber of trials: %d\n' % sim.trials)

    list_bumps_and_energies(file, sim)

    log('\tHBond Minimum Window Size: %d\n' % sim.hbond_winmin)
    log('\tHBond Maximum Window Size: %d\n' % sim.hbond_winmax)

    log('\tContact Minimum Window Size: %d\n' % sim.contact_winmin)
    log('\tContact Maximum Window Size: %d\n' % sim.contact_winmax)


    log('\n----- Sampling Weights at Start of Run -----\n')

    hw = sim.protein.residue_helix_weights
    sw = sim.protein.residue_strand_weights
    tw = sim.protein.residue_turn1_weights
    Tw = sim.protein.residue_turn2_weights
    cw = sim.protein.residue_coil_weights
    pw = sim.protein.residue_PII_weights
    rnames = sim.protein.residue_names

    log('RESIDUE    HELIX   SHEET   TURN1   TURN2    COIL    PII\n')

    format = '%4d %3s%8.3f%8.3f%8.3f%8.3f%8.3f%8.3f\n'
    for i in range(sim.protein.num_residues):
        log(format % (i+1, rnames[i], hw[i], sw[i], tw[i], Tw[i], cw[i], pw[i]))

        


def writeepilogue(file, sim, nacp, nrej, nrejs, nrejhbs, eref, ebest):
    log = file.write
    log('Simulation Finished at: %s\n' % time.ctime(time.time()))
    log('Number of Structures Accepted: %d\n' % nacp)
    log('Number of Structures Rejected by Sterics: %d\n' % nrejs)
    log('Number of Structures Rejected by Hbond Satisfaction: %d\n' % nrejhbs)
    log('Total Number of Structures Rejected: %d\n' % (nrej+nrejs+nrejhbs,))
    log('Best Score: %g\n' % ebest)
    log('Final Score: %g\n' % eref)
    log('Random Generator Seeds at End: %s\n' % `sim.generator.get_seed()`)

    list_bumps_and_energies(file, sim)

    log('\n----- Sampling Weights at Start of Run -----\n')
    # move attempt statistics
    rnames = sim.protein.residue_names

    log('\n----- Move Type Details ----\n')
    log('  Residue        Helix               Sheet              Turn1        \
      Turn2               Coil                PII\n')
    log('           Accepted Rejected   Accepted Rejected  Accepted Rejected  \
Accepted Rejected  Accepted Rejected  Accepted Rejected\n')

    fmt = ' %4d %3s  %8d %8d  %8d %8d  %8d %8d  %8d %8d  %8d %8d  %8d %8d\n'
    for i in range(sim.protein.num_residues):
        row = sim.move_types_attempted[i]
        row = (i+1, rnames[i]) + tuple(row[0] + row[1] + row[2] + row[3] + row[4] + row[5])
        log(fmt % row)




def compute_weights(sslist):
    row = sslist[0]
    denom = 0
    for i in row:
        denom = denom + i
    denom = 1.0 / denom
    w = [None]*len(sslist)
    pos = 0
    for h, s, t1, t2, c, p in sslist:
        w[pos] = [h*denom, s*denom, t1*denom, t2*denom, c*denom, p*denom]
        pos = pos + 1
    return w

def write_swt(file, rnames, w):
    if type(file) is type(''):
        ff = open(file, 'wb')
    else:
        ff = file

    ff.write('='*60 + '\n')
    ff.write(' Conformation Weights \n')
    ff.write('='*60 + '\n')
    ff.write('RESIDUE    HELIX   SHEET   TURN1   TURN2    COIL    PII\n')
    format = '%4d %3s%8.3f%8.3f%8.3f%8.3f%8.3f%8.3f\n'

    pos = 1
    for h, s, t1, t2, c, p in w:
        ff.write(format % (pos, rnames[pos-1], h, s, t1, t2, c, p))
        pos = pos + 1

    if ff <> file:
        ff.close()

def plot_swt(file, w, title):
    try:
        from Plot import swtplotPII
    except ImportError:
        pass
    else:
        swtplotPII.plot(w, file, title)
    return
    
    
