import math
NORM = 2.328306549295728e-10
M1   = 4294967087.0
M2   = 4294944443.0
A12  =    1403580.0
A13N =     810728.0
A21  =     527612.0
A23N =    1370589.0

class _CMRG32:

    def __init__(self, seeds):
        self.seeds = seeds

    def random(self):
        s = self.seeds
        p1 = A12 * s[1] - A13N * s[0]
        k = int(p1 / M1)
        p1 = p1 - k * M1
        if p1 < 0.0: p1 = p1 + M1
        s[0] = s[1]
        s[1] = s[2]
        s[2] = p1

        p2 = A21 * s[5] - A23N * s[3]
        k  = int(p2 / M2)
        p2 = p2 - k * M2
        if p2 < 0.0: p2 = p2 + M2
        s[3] = s[4]
        s[4] = s[5]
        s[5] = p2

        if p1 <= p2:
            return ((p1 - p2 + M1) * NORM)
        else:
            return ((p1 - p2) * NORM)

    def sample(self, n):
        ret = range(n)
        random = self.random
        for i in ret:
            ret[i] = random()
        return ret

    def choice(self, seq):
        n = len(seq)
        r = int(n * self.random())
        return seq[r]

    def uniform(self, low, high):
        diff = high - low
        rv = self.random()
        prod = rv * diff
        return low + prod

    def savestate(self, file):
        if type(file) is type(''):
            ff = open(file, 'wb')
        else:
            ff = file
        
        for seed in self.seeds:
            ff.write('%g\n' % seed)

        if ff <> file: ff.close()

    def getstate(self, file):
        if type(file) is type(''):
            ff = open(file, 'rb')
        else:
            ff = file

        for i in range(6):
            self.seeds[i] = float(ff.readline())

        if ff <> file: ff.close()

    def get_seed(self): return tuple(self.seeds)

    def randint(self, low, high):
        rv = self.random()
        return int(low + (rv*(high - low + 1)))

    def permutation(self, seq):
        ret = list(seq)
        n = len(seq)
        for i in range(n):
            rv = self.random()
            pos = int(rv * n)
            ret[i], ret[pos] = ret[pos], ret[i]
        return ret
        

def generator(s1, s2, s3, s4, s5, s6):
    if s1 == s2 == s3 == 0:
        raise ValueError, 'First three seeds are all zero'
    if s4 == s4 == s6 == 0:
        raise ValueError, 'Last three seeds are all zero'
    if (s1 < 0 or s1 >=M1) or (s2 < 0 or s2 >= M1) or (s3 < 0 or s3 >= M1):
        raise ValueError, 'First three seeds must be in range 0 to 4294967087'
    if (s4 < 0 or s4 >=M2) or (s5 < 0 or s5 >= M2) or (s6 < 0 or s6 >= M2):
        raise ValueError, 'Last three seeds must be in range 0 to 4294944443'
    return _CMRG32([s1, s2, s3, s4, s5, s6])

try:
    from cmrg32 import generator
except ImportError:
    print 'Using slower python version of random number generator'
    pass

