from pro_dic import *
from mol import *
import string
import Atom3d
import prosst

class Protein(Chain):

    def clean(self):
        res = self.elements
        for i in self.reverse_indices():
            tmp = len(res[i].atoms_with_name('N', 'CA', 'C'))
            if tmp <> 3:
                del res[i]


    def gaps(self):
        residues = self.elements
        dlist = []
        for i in xrange(len(self)-2):
            ca1 = residues[i].atoms_with_name('CA')
            ca2 = residues[i+1].atoms_with_name('CA')
            if not ca1 or not ca2:
                if hasattr(residues[i+1], 'het'): continue
                self[i].gap = 1
            else:
                d = ca1[0].sqr_distance(ca2[0]) 
                if d > 18:
                    residues[i].gap = 1
                elif d <= 0.1:
                    dlist.append(i+1)
        dlist.reverse()
        for i in dlist:
            del residues[i]

    def phi(self, i):
        res = self.elements[i]       
        if i==0 or hasattr(self.elements[i-1], 'gap'):
            return 999.99

        try:
            a, b, c = res.atoms_with_name('N', 'CA', 'C')
        except:
            return 999.99

        try:
            prev = self[i-1].atoms_with_name('C')[0]
        except IndexError:
            return 999.99
        else:
            return prev.torsion(a, b, c)

    def psi(self, i):
        res = self.elements[i]       
        if i >= len(self)-1 or hasattr(res, 'gap'):
            return 999.99

        try:
            a, b, c = res.atoms_with_name('N', 'CA', 'C')
        except:
            return 999.99

        try:
            next = self[i+1].atoms_with_name('N')[0]
        except IndexError:
            return 999.99
        else:
            return a.torsion(b, c, next)

    def omega(self, i):
        res = self.elements[i]       
        if i >= len(self)-1 or hasattr(res, 'gap'):
            return 999.99

        try:
            a, b = res.atoms_with_name('CA', 'C')
        except:
            return 999.99

        try:
            c, d = self[i+1].atoms_with_name('N', 'CA')
        except:
            return 999.99

        return a.torsion(b, c, d)

    def chi1(self, i):
        res = self.elements[i]       
        atoms = CHI1_ATOMS.get(res.name, None)
        if atoms is None:
            return 999.99
        
        try:
            a, b, c, d = apply(res.atoms_with_name, atoms)
        except:
            return 999.99
        else:
            return a.torsion(b, c, d)

    def chi2(self, i):
        res = self.elements[i]       
        atoms = CHI2_ATOMS.get(res.name, None)
        if atoms is None:
            return 999.99
        
        try:
            a, b, c, d = apply(res.atoms_with_name, atoms)
        except:
            return 999.99
        else:
            return a.torsion(b, c, d)

    def chi3(self, i):
        res = self.elements[i]       
        atoms = CHI3_ATOMS.get(res.name, None)
        if atoms is None:
            return 999.99
        
        try:
            a, b, c, d = apply(res.atoms_with_name, atoms)
        except:
            return 999.99
        else:
            return a.torsion(b, c, d)

    def chi4(self, i):
        res = self.elements[i]       
        atoms = CHI4_ATOMS.get(res.name, None)
        if atoms is None:
            return 999.99
        
        try:
            a, b, c, d = apply(res.atoms_with_name, atoms)
        except:
            return 999.99
        else:
            return a.torsion(b, c, d)

    def backbone_atoms(self, *ids):
        if not len(ids):
            ids = xrange(len(self))

        bb_atoms = ('N', 'CA', 'C', 'O')
        residues = []
        Append = residues.append
        res_class = res.__class__
        Strip = string.strip

        for i in ids:
            res = self[i]
            atoms = []
            for atom in res:
                if atom.name in bb_atoms: atoms.append(atom)
            new = res_class(res.name)
            res.elements = atoms
            Append(res)
        new_chain = self.__class__(self.name)
        new_chain.elements = residues
        return new_chain

    def torsions(self, i=None, map=map):
        if not i is None:
            return self.phi(i), self.psi(i), self.omega(i), self.chi1(i),\
                   self.chi2(i), self.chi3(i), self.chi4(i)
        else:
            n = range(len(self))
            return map(self.phi, n),\
                   map(self.psi, n),\
                   map(self.omega, n),\
                   map(self.chi1, n),\
                   map(self.chi2, n),\
                   map(self.chi3, n),\
                   map(self.chi4, n)


    def add_backbone_hydrogens(self):
        """Adds backbone polar hydrogens to a peptide chain"""
        return prosst.add_backbone_hydrogens(self)

    def calc_hbonds(self):
        return prosst.calc_hbonds(self)

    def dssp(self, phi=None, psi=None, ome=None):
        return prosst.dssp(self, phi, psi, ome)

    def pross(self, phi=None, psi=None, ome=None):
        return prosst.rc_ss(self, phi, psi, ome)             
        
    def type(self):
        return 'protein'

    def normalized_area(self, probe=1.4, ndiv=3):
        self.surf_vol(probe, ndiv)
        area_dict = residueAreas
        for res in self.elements:
            try:
                val = area_dict[res.name]
            except KeyError:
                res.norm_area = 1.0
            else:
                res.norm_area = res.sasa/val['SS']

    def sequence(self, one=0):
        n = self.num_residues()
        seq = map(getattr, self.elements, ('name',)*n)
        if one:
            get_one = ONE_LETTER_CODES.get
            seq = map(get_one, seq, 'X'*n)
        return seq
            

class AminoAcid(Residue):

    def chi1(self):
        atoms = CHI1_ATOMS.get(self.name, None)
        if atoms is None:
            return 999.99
        
        try:
            a, b, c, d = apply(self.atoms_with_name, atoms)
        except:
            return 999.99
        else:
            return a.torsion(b, c, d)

    def chi2(self):
        atoms = CHI2_ATOMS.get(self.name, None)
        if atoms is None:
            return 999.99
        
        try:
            a, b, c, d = apply(self.atoms_with_name, atoms)
        except:
            return 999.99
        else:
            return a.torsion(b, c, d)

    def chi3(self):
        atoms = CHI3_ATOMS.get(self.name, None)
        if atoms is None:
            return 999.99
        
        try:
            a, b, c, d = apply(self.atoms_with_name, atoms)
        except:
            return 999.99
        else:
            return a.torsion(b, c, d)

    def chi4(self):
        atoms = CHI4_ATOMS.get(self.name, None)
        if atoms is None:
            return 999.99
        
        try:
            a, b, c, d = apply(self.atoms_with_name, atoms)
        except:
            return 999.99
        else:
            return a.torsion(b, c, d)

    def assign_radii(self):
        d_get = SASARAD[self.name].get
        for atom in self.elements:
            atom.radius = d_get(atom.name,0.0)
            

    
    
